#include "stdafx.h"
#include "gen_typs.h"
#include "osd_exp.h"

#if defined( ESTIMATE_CODE_SIZE )
#pragma code_seg( ".eng_code" )
#endif

#ifndef _X8616B
	#if !defined( DT64BSUPPORTED )
		const U64 BASEATTR U64_ZERO = { 0L,0L };
	#elif defined( _SHASTAFW )
		const U64 BASEATTR U64_ZERO = { 0 };
	#else
		const U64 BASEATTR U64_ZERO = { { 0L, 0L } };
	#endif
#endif	/* _X8616B */

/*
 =========================================================================================
    Note that following 64 bit arithmetic implement for compilers that doesn't support 64
    bit integer is limited. Actually only 32 bit is supported. Please replace this
    implementation in the future.
 =========================================================================================
 */
U64
U64_Add( U64 addend1, U64 addend2 )
{
	#if defined( DT64BSUPPORTED )
	addend1.Value += addend2.Value;
	#else
	addend1.u.Low += addend2.u.Low;
	#endif
	return addend1;
}

U64
U64_Add_U32( U64 addend1, U32 addend2 )
{
	#if defined( DT64BSUPPORTED )
	addend1.Value += addend2;
	#else
	addend1.u.Low += addend2;
	#endif
	return addend1;
}

U64
U64_Subtract( U64 Minuend, U64 Subtrahend )
{
	#if defined( DT64BSUPPORTED )
	Minuend.Value -= Subtrahend.Value;
	#else
	Minuend.u.Low -= Subtrahend.u.Low;
	#endif
	return Minuend;
}

U64
U64_Subtract_U32( U64 Minuend, U32 Subtrahend )
{
	#if defined( DT64BSUPPORTED )
	Minuend.Value -= Subtrahend;
	#else
	Minuend.u.Low -= Subtrahend;
	#endif
	return Minuend;
}

#if defined(_OS_LINUX) 

#if !defined(_X8632B)
#include <asm/div64.h>
#endif

/* isaac add for PCM using */
_U64
u64_divide_u32(_U64 Dividend, U32 Divisor, U32* Remainder)
{
	_U64 result = Dividend;
	U32 tmp = do_div(result, Divisor);

	if( Remainder )
		*Remainder = tmp;	
	
	return result;
}

U64
U64_Divide( U64 Dividend, U64 Divisor, U64* Remainder )
{
	U64 result = Dividend;
	U32 tmp = do_div(result.Value, Divisor.u.Low);
	if( Remainder ){
		Remainder->u.High = 0;
		Remainder->u.Low = tmp;
	}
	return result;
}

U64
U64_Divide_U32( U64 Dividend, U32 Divisor, U32* Remainder )
{
	U64 result = Dividend;
	U32 tmp = do_div(result.Value, Divisor);
	 
	if(Remainder)	
		*Remainder = tmp;
	
	return result;
}

#else	/* _OS_LINUX */

#if defined(DT64BSUPPORTED)
_U64
u64_divide_u32(_U64 Dividend, U32 Divisor, U32* Remainder)
{
	if( Remainder )
		*Remainder = ( U32 ) ( Dividend % Divisor );
	Dividend /= Divisor;

	return Dividend;

}
#endif

U64
U64_Divide( U64 Dividend, U64 Divisor, U64* Remainder )
{
	#if defined( DT64BSUPPORTED )
	if( Remainder )
		Remainder->Value = Dividend.Value % Divisor.Value;
	Dividend.Value /= Divisor.Value;
	#else
	if( Remainder )
	{
		Remainder->u.High = 0;
		Remainder->u.Low = Dividend.u.Low % Divisor.u.Low;
	}

	Dividend.u.Low /= Divisor.u.Low;
	#endif
	return Dividend;
}

U64
U64_Divide_U32( U64 Dividend, U32 Divisor, U32* Remainder )
{
	#if defined( DT64BSUPPORTED )
	if( Remainder )
		*Remainder = ( U32 ) ( Dividend.Value % Divisor );
	Dividend.Value /= Divisor;
	#else
	if( Remainder )
		*Remainder = Dividend.u.Low % Divisor;
	Dividend.u.Low /= Divisor;
	#endif
	return Dividend;
}
#endif

U64
U64_Multiply_U32( U64 Multiplicand, U32 Multiplier )
{
	#if defined( DT64BSUPPORTED )
	Multiplicand.Value *= Multiplier;
	#else
	Multiplicand.u.Low *= Multiplier;
	#endif
	return Multiplicand;
}

U32
U64_2_U32( U64 value )
{
	if( value.u.High == 0 )
		return value.u.Low;
	return U32_MAX;
}

I32
U64_Compare( U64 value1, U64 value2 )
{
	U32 diff;

	if( value1.u.High > value2.u.High )
		return 1;
	else if( value1.u.High < value2.u.High )
		return -1;

	/* value1.u.High==value2.u.High */

	/*
	This result signed/unsigned conversion problem when difference between
	value1.u.Low and value2.u.Low is bigger than 0x80000000.
	*/
	/*
	return value1.u.Low - value2.u.Low;
	*/

	if( value1.u.Low>value2.u.Low )
	{
		diff = value1.u.Low - value2.u.Low;
		return (I32) Min( diff, I32_MAX );
	}
	else if( value1.u.Low<value2.u.Low )
	{
		diff = value2.u.Low - value1.u.Low;
		return - (I32) Min( diff, I32_MAX );
	}
	else
		return 0;
}

#if !defined(_X8616B) && !defined(SIMULATE_BIOS_ENV)
/* U64_Abs_Diff is not required for BIOS */
U64
U64_Abs_Diff( U64 value1, U64 value2 )
{
	U64 ret;
	U64_Zerolize( ret );

	#if defined( DT64BSUPPORTED )
	ret.Value = ( (value1.Value > value2.Value) ? (value1.Value - value2.Value) : (value2.Value - value1.Value) );
	#else
	ret.u.Low =
		(
			(value1.u.Low > value2.u.Low) ? (value1.u.Low - value2.u.Low) :
				(value2.u.Low - value1.u.Low)
		);
	#endif
	return ret;
}
#endif

#if !defined(_X8616B)
BoOL SGTableAppendItemEx(
	bpSGTABLE	pSgTable,
	U32			StartAddressLo,
	U32			StartAddressHi,
	U32			ByteLength,
	U8			CtrlFlags )
{
	bpSG	p_sg;

	if( pSgTable->SGCount + 1 > pSgTable->MaxSGCount )
	{
		OSD_ASSERT( FALSE );
		return FALSE;
	}

	if( pSgTable->SGCount )
	{
		p_sg = &pSgTable->table[pSgTable->SGCount - 1];
		p_sg->CtrlFlag &= ~SG_CF_EOT;
	}

	pSgTable->SGCount++;
	p_sg = &pSgTable->table[pSgTable->SGCount - 1];

	p_sg->CtrlFlag = (U8) ( CtrlFlags | SG_CF_EOT );
	p_sg->Count = ByteLength;
#if defined(USE_32BIT_SG_DEFAULT)
	p_sg->StartAddress = StartAddressLo;
#else
	p_sg->StartAddress.u.Low = StartAddressLo;
	p_sg->StartAddress.u.High = StartAddressHi;
#endif

	pSgTable->SizeInByte += ByteLength;

	return TRUE;
}
#endif /* _X8616B */

BoOL
SGTableAppendItem(
	bpSGTABLE	pSgTable,
	PTR_SIZE	StartAddress,
	U32			ByteLength,
	U8			CtrlFlags )
{
	bpSG	p_sg;
	BoOL	bNewEntry = TRUE;
	/* merge code from Ranger Yu to merge sgtable if contiguous */

	if( pSgTable->SGCount + 1 > pSgTable->MaxSGCount )
	{
		OSD_ASSERT( FALSE );
		return FALSE;
	}

	if( pSgTable->SGCount )
	{
		p_sg = &pSgTable->table[pSgTable->SGCount - 1];
#if MAX_SG_ITEM_PER_TABLE < 8
		/* I am still hesitating to add this to generic versions without more tests. */
		if( SG_STARTADDRESS(p_sg->StartAddress)+p_sg->Count==StartAddress )
			bNewEntry = FALSE;
#endif
		p_sg->CtrlFlag &= ~SG_CF_EOT;
	}

	if( bNewEntry )
	{
		pSgTable->SGCount++;
		p_sg = &pSgTable->table[pSgTable->SGCount - 1];

		p_sg->CtrlFlag = (U8) ( CtrlFlags | SG_CF_EOT );
		p_sg->Count = ByteLength;

		SG_ASSIGN_STARTADDRESS( p_sg->StartAddress, StartAddress );
	}
	else
	{
		p_sg->Count += ByteLength;
		p_sg->CtrlFlag |= SG_CF_EOT;
	}

	pSgTable->SizeInByte += ByteLength;

	return TRUE;
}
